@@ -26,6 +26,10 @@
 #include <errno.h>
 #include <unistd.h>
 #include <string.h>
+#ifdef HAVE_ALLOCA_H
+#include <alloca.h>
+#endif /* HAVE_ALLOCA_H */
+#include <stdlib.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <netinet/in.h>
@@ -40,6 +44,7 @@
 #ifdef HAVE_SHADOW_H
 #include <shadow.h>
 #endif
+#include "compat.h"
 
 #ifndef CHECKROOT
 /* Set to 0 if you don't want to check against the root password
@@ -187,7 +192,7 @@ yppasswdproc_pwupdate_1 (yppasswd *yppw, struct svc_req *rqstp)
   int retries;
   static int res;                /* I hate static variables */
   char *logbuf;
-  struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
+  const struct sockaddr_in *rqhost = svc_getcaller (rqstp->rq_xprt);
 
   /* Be careful here with the debug option. You can see the old
      and new password in clear text !! */
@@ -276,6 +281,7 @@ yppasswdproc_pwupdate_1 (yppasswd *yppw, struct svc_req *rqstp)
     }
   else
     {
+#ifdef HAVE_LCKPWDF
       /* Lock the passwd file. We retry several times. */
       retries = 0;
       while (lckpwdf () && retries < MAX_RETRIES)
@@ -290,11 +296,14 @@ yppasswdproc_pwupdate_1 (yppasswd *yppw, struct svc_req *rqstp)
 	  log_msg ("password file locked");
 	  return &res;
 	}
+#endif /* HAVE_LCKPWDF */
 
       res = update_files (yppw, logbuf, &shadow_changed, &passwd_changed,
 			  &chfn, &chsh);
 
+#ifdef HAVE_LCKPWDF
       ulckpwdf ();
+#endif /* HAVE_LCKPWDF */
     }
 
   /* Fork off process to rebuild NIS passwd.* maps. */
@@ -358,6 +367,7 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
     {
       if (strcmp (pw->pw_passwd, "x") == 0)
 	{
+#ifdef HAVE_GETSPNAM /* shadow password */
 	  struct spwd *spw;
 
 	  if ((spw = getspnam ("root")) != NULL)
@@ -365,6 +375,7 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
 	      rootpass = alloca (strlen (spw->sp_pwdp) + 1);
 	      strcpy (rootpass, spw->sp_pwdp);
 	    }
+#endif /* HAVE_GETSPNAM */
 	}
       else
 	{
@@ -402,6 +413,7 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
   chmod (path_passwd_tmp, passwd_stat.st_mode);
   chown (path_passwd_tmp, passwd_stat.st_uid, passwd_stat.st_gid);
 
+#ifdef HAVE_GETSPNAM
   /* Open the shadow file for reading. */
   if ((oldsf = fopen (path_shadow, "r")) != NULL)
     {
@@ -426,6 +438,7 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
       chmod (path_shadow_tmp, shadow_stat.st_mode);
       chown (path_shadow_tmp, shadow_stat.st_uid, shadow_stat.st_gid);
     }
+#endif /* HAVE_GETSPNAM */
 
   /* Loop over all passwd entries */
   while ((pw = fgetpwent (oldpf)) != NULL)
@@ -442,6 +455,7 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
 	  if (oldsf != NULL &&
 	      pw->pw_passwd[0] == 'x' && pw->pw_passwd[1] == '\0')
 	    {
+#ifdef HAVE_GETSPNAM /* shadow password */
 	      /* Search for the shadow entry of this user */
 	      while ((spw = fgetspent (oldsf)) != NULL)
 		{
@@ -463,6 +477,7 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
 		      goto error;
 		    }
 		}
+#endif /* HAVE_GETSPNAM */
 	    }
 
 	  /* We don't have a shadow password file or we don't find the
@@ -483,6 +498,7 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
 		yppw->newpw.pw_passwd[1] == '\0') &&
 	      yppw->newpw.pw_passwd[0] != '\0')
 	    {
+#ifdef HAVE_GETSPNAM /* shadow password */
 	      if (spw)
 		{
 		  /* test if password is expired */
@@ -525,6 +541,7 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
 		      }
 		}
 	      else /* No shadow entry */
+#endif /* HAVE_GETSPNAM */
 		{
 		  /* set the new passwd */
 		  pw->pw_passwd = yppw->newpw.pw_passwd;
@@ -593,9 +610,12 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
   if (pw || spw)
     {
       unlink (path_passwd_tmp);
+#ifdef HAVE_GETSPNAM
       unlink (path_shadow_tmp);
+#endif /* HAVE_GETSPNAM */
       return 1;
     }
+#ifdef HAVE_GETSPNAM
   if (*shadow_changed)
     {
       unlink (path_shadow_old);
@@ -604,6 +624,7 @@ update_files (yppasswd *yppw, char *logbuf, int *shadow_changed,
     }
   else
     unlink (path_shadow_tmp);
+#endif /* HAVE_GETSPNAM */
 
   if (*passwd_changed)
     {
@@ -634,7 +655,13 @@ external_update_env (yppasswd *yppw)
     }
   else
     { /* Child - run external update program */
-#if (defined(__sun__) || defined(sun)) && defined(__svr4__)
+#if defined(HAVE_SETENV)
+      setenv ("YP_PASSWD_OLD", yppw->oldpass, 1);
+      setenv ("YP_PASSWD_NEW", yppw->newpw.pw_passwd, 1);
+      setenv ("YP_USER", yppw->newpw.pw_name, 1);
+      setenv ("YP_GECOS", yppw->newpw.pw_gecos, 1);
+      setenv ("YP_SHELL", yppw->newpw.pw_shell, 1);
+#elif defined(HAVE_PUTENV)
       char aenv[5*64], *p;
       p = aenv;
       sprintf(p, "YP_PASSWD_OLD=%s", yppw->oldpass);
@@ -652,11 +679,7 @@ external_update_env (yppasswd *yppw)
       sprintf(p, "YP_SHELL=%s", yppw->newpw.pw_shell);
       putenv(p);
 #else
-      setenv ("YP_PASSWD_OLD", yppw->oldpass, 1);
-      setenv ("YP_PASSWD_NEW", yppw->newpw.pw_passwd, 1);
-      setenv ("YP_USER", yppw->newpw.pw_name, 1);
-      setenv ("YP_GECOS", yppw->newpw.pw_gecos, 1);
-      setenv ("YP_SHELL", yppw->newpw.pw_shell, 1);
+#  error "Missing both setenv() and putenv().  Need porting."
 #endif
       execlp (external_update_program, external_update_program, NULL);
       _exit (1); /* fall-through */
