@@ -34,6 +34,7 @@
 #include <fcntl.h>
 #include <syslog.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 #include <rpc/rpc.h>
@@ -42,21 +43,12 @@
 #include <rpc/svc_soc.h>
 #endif
 #include "yppasswd.h"
-#if defined(HAVE_GETOPT_H) && defined(HAVE_GETOPT_LONG)
+#if defined(HAVE_GETOPT_H)
 #include <getopt.h>
-#else
-#include <compat/getopt.h>
-#endif
-#ifndef HAVE_GETOPT_LONG
-#include <compat/getopt.c>
-#include <compat/getopt1.c>
-#endif
-
-#ifndef HAVE_STRERROR
-#include <compat/strerror.c>
 #endif
 
 #include "log_msg.h"
+#include "compat.h"
 
 #ifdef HAVE_PATHS_H
 #include <paths.h>
@@ -197,14 +189,14 @@ usage (FILE * fp, int n)
 }
 
 static void
-sig_child (int sig __attribute__ ((unused)))
+sig_child (int sig UNUSED)
 {
   wait (NULL);
 }
 
 /* Clean up if we quit the program. */
 static void
-sig_quit (int sig __attribute__ ((unused)))
+sig_quit (int sig UNUSED)
 {
   pmap_unset (YPPASSWDPROG, YPPASSWDVERS);
   unlink (_YPPASSWDD_PIDFILE);
