@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2001, 2002  Thorsten Kukuk
+/* Copyright (c) 2000, 2001, 2002, 2003, 2004  Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    The YP Server is free software; you can redistribute it and/or
@@ -23,20 +23,30 @@
 
 #include <unistd.h>
 #include <syslog.h>
+#if defined(HAVE_GETOPT_H)
 #include <getopt.h>
+#endif /* HAVE_GETOPT_H */
 #include <netdb.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <time.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <ctype.h>
+#ifdef HAVE_ALLOCA_H
+#include <alloca.h>
+#endif /* HAVE_ALLOCA_H */
+#include <stdlib.h>
+#include <sys/stat.h>
+#include <sys/param.h>
 #include "log_msg.h"
 #include "yp.h"
 #include "ypxfr.h"
 #include "ypxfrd.h"
 #include <rpcsvc/ypclnt.h>
+#include "compat.h"
 
 #if defined(HAVE_LIBGDBM)
 #include <gdbm.h>
@@ -188,7 +198,7 @@ xdr_ypxfr_xfr (XDR *xdrs, xfr *objp)
     }
 }
 
-int
+static int
 ypxfrd_transfer (char *host, char *map, char *domain, char *tmpname)
 {
   CLIENT *clnt;
@@ -276,11 +286,11 @@ error:
 #if defined(__NetBSD__)
 static int
 ypxfr_foreach (u_long status, char *key, int keylen,
-               char *val, int vallen, void *data __attribute__ ((unused)))
+               char *val, int vallen, void *data UNUSED)
 #else
 static int
 ypxfr_foreach (int status, char *key, int keylen,
-               char *val, int vallen, char *data __attribute__ ((unused)))
+               char *val, int vallen, char *data UNUSED)
 #endif
 {
   datum outKey, outData;
@@ -345,8 +355,9 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
   CLIENT *clnt_udp;
   struct sockaddr_in sockaddr, sockaddr_udp;
   struct ypresp_order resp_order;
+  struct ypresp_master resp_master;
   struct ypreq_nokey req_nokey;
-  uint32_t masterOrderNum;
+  time_t masterOrderNum;
   struct hostent *h;
   int sock, result;
 
@@ -377,8 +388,6 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
       h = gethostbyname (source_host);
       if (!h)
 	return YPXFR_RSRC;
-      master_host = alloca (strlen (source_host) + 1);
-      strcpy (master_host, source_host);
     }
   else
     {
@@ -388,10 +397,8 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
 
       if (yp_master (source_domain, map, &master_name))
         return YPXFR_MADDR;
-      master_host = alloca (strlen (master_name) + 1);
-      strcpy (master_host, master_name);
+      h = gethostbyname (master_name);
       free (master_name);
-      h = gethostbyname (master_host);
       if (!h)
 	return YPXFR_RSRC;
     }
@@ -407,6 +414,42 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
       return YPXFR_RPC;
     }
 
+  /* We cannot use the libc functions since we don't know which host
+     they use. So query the host we must use to get the official master
+     server name for the map on the master host. */
+  req_nokey.domain = source_domain;
+  req_nokey.map = map;
+  if (ypproc_master_2 (&req_nokey, &resp_master, clnt_udp) != RPC_SUCCESS)
+    {
+      log_msg (clnt_sperror (clnt_udp, "ypproc_master_2"));
+      return YPXFR_YPERR;
+    }
+  else if (resp_master.stat != YP_TRUE)
+    {
+      switch (resp_master.stat)
+	{
+	case YP_NOMAP:
+	  return YPXFR_NOMAP;
+	case YP_NODOM:
+	  return YPXFR_NODOM;
+	case YP_BADDB:
+	  return YPXFR_DBM;
+	case YP_YPERR:
+	  return YPXFR_YPERR;
+	case YP_BADARGS:
+	  return YPXFR_BADARGS;
+	default:
+	  log_msg ("ERROR: not expected value: %s", resp_master.stat);
+	  return YPXFR_XFRERR;
+	}
+    }
+  else
+    {
+      master_host = alloca (strlen (resp_master.peer) + 1);
+      strcpy (master_host, resp_master.peer);
+      xdr_free ((xdrproc_t) xdr_ypresp_master, (caddr_t) &resp_master);
+    }
+
   /* We cannot use the libc functions since we don't know which host
      they use. So query the host we must use to get the order number
      for the map on the master host. */
@@ -414,7 +457,7 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
   req_nokey.map = map;
   if (ypproc_order_2 (&req_nokey, &resp_order, clnt_udp) != RPC_SUCCESS)
     {
-      log_msg (clnt_sperror (clnt_udp, "masterOrderNum"));
+      log_msg (clnt_sperror (clnt_udp, "ypproc_order_2"));
       masterOrderNum = time (NULL); /* We set it to the current time.
                                        So a new map will be always newer. */
     }
@@ -440,13 +483,13 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
   else
     {
       masterOrderNum = resp_order.ordernum;
-      xdr_free ((xdrproc_t) xdr_ypresp_order, (char *) &resp_order);
+      xdr_free ((xdrproc_t) xdr_ypresp_order, (caddr_t) &resp_order);
     }
 
   /* If we doesn't force the map, look, if the new map is really newer */
   if (!force)
     {
-      uint32_t localOrderNum = 0;
+      time_t localOrderNum = 0;
       datum inKey, inVal;
 
 #if defined(HAVE_LIBGDBM)
@@ -532,7 +575,7 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
           unlink (dbName_temp);
           return YPXFR_DBM;
         }
-      snprintf (orderNum, sizeof (orderNum), "%d", masterOrderNum);
+      snprintf (orderNum, sizeof (orderNum), "%ld", (long)masterOrderNum);
       outKey.dptr = "YP_LAST_MODIFIED";
       outKey.dsize = strlen (outKey.dptr);
       outData.dptr = orderNum;
@@ -684,7 +727,7 @@ main (int argc, char **argv)
   struct in_addr remote_addr;
   unsigned int transid = 0;
   unsigned short int remote_port = 0;
-  uint32_t program_number = 0;
+  unsigned long program_number = 0;
   int force = 0;
   int noclear = 0;
 
@@ -774,10 +817,18 @@ main (int argc, char **argv)
   argv += optind;
 
   if (target_domain == NULL)
-    yp_get_default_domain (&target_domain);
+    {
+      yp_get_default_domain (&target_domain);
+
+      if (target_domain == NULL)
+	{
+	  log_msg ("Please specify domainname");
+	  exit (1);
+	}
+    }
 
   if (source_domain == NULL)
-    source_domain = target_domain;
+      source_domain = target_domain;
 
   for (; *argv; argv++)
     {
